/*
* cleint-interface for JVisu socket server
*
* Copyright (C) 2003 Thomas Gleixner <tglx@linutronix.de>
*		
* PURPOSE
* Simple interface to JVisu socket server. Used from scripts on the
* local machine to read / modify variables
*/

/*
* LICENCING INFORMATION
*
* JVisu is free software; you can redistribute it and/or modify it under
* the terms of the GNU General Public License version 2 as published by the
* Free Software Foundation.
*
* JVisu is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along
* with JVisu; if not, write to the Free Software Foundation, Inc.,
* 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
*/

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <limits.h>
#include <time.h>
#include <getopt.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <netdb.h>
#include <jvisu.h>

char	answer[4096];
char	symbol[256];
char	value[256];
char	writevalue[256];
int	port = 2201;
int	writevar;
int	vartype = 1;


/** Display the usage information */
void display_help (void)
{
	printf("Usage: jvisuclient [OPTIONS]\n"
	       "short description.\n"
	       "\n"
	       "  -s, --symbol sym	variable symbol\n"
	       "  -p, --port port 	portnumber to connect (default: 2201)\n"
	       "  -t, --type type	variable type\n"
	       "  -w, --write		write value given as last argument\n"
	       "      --help     	display this help and exit\n"
	       "      --version  	output version information and exit\n");
	exit(0);
}

/** Display the version information */
void display_version (void)
{
	printf ("jvisuclient V1.0\n");
	exit (0);
}


/** Process commandline options */
void process_options (int argc, char *argv[])
{
	int error = 0;

	for (;;) {
		int option_index = 0;
		static const char *short_options = "p:s:t:w";
		static const struct option long_options[] = {
			{"help", no_argument, 0, 0},
			{"version", no_argument, 0, 0},
			{"port", required_argument, 0, 'p'},
			{"symbol", required_argument, 0, 's'},
			{"type", required_argument, 0, 't'},
			{"write", no_argument, 0, 'w'},
			{0, 0, 0, 0},
		};

		int c = getopt_long(argc, argv, short_options,
				    long_options, &option_index);
		if (c == EOF)
			break;
		
		switch (c) {
		/* Option without short option */
		case 0:
			switch (option_index) {
			case 0:
				display_help();
				break;
			case 1:
				display_version();
				break;
			}
			break;
		/* Short options */	
		case 'p':
			port = atoi (optarg);
			break;
			
		case 's':
			strcpy (symbol, optarg);
			break;

		case 't':
			vartype = atoi (optarg);
			break;

		case 'w':
			writevar = 1;
			break;
				
		case '?':
			error = 1;
			break;
		}
	}
	
	if ((argc - optind) != writevar || error) 
		display_help ();
		
	if (writevar)
		strcpy (writevalue, argv[argc - 1]);		
}

static int read_result (int sock)
{
	fd_set			rfds;
	struct timeval		timeo;
	char			buf[256];
	struct jvisu_sndvar	*rec = (struct jvisu_sndvar *) buf;
	int			len;
	
	memset (buf, 0, sizeof (buf));
	// Read the result
	FD_SET (sock, &rfds);
	timeo.tv_sec = 1;
	timeo.tv_usec = 0;
	if (select(sock + 1, &rfds, NULL, NULL, &timeo) <= 0) {
		fprintf (stderr, "Select failed\n");
		return -1;
	}	
	if (read (sock, buf, sizeof (struct jvisu_cmdhdr)) != sizeof (struct jvisu_cmdhdr)) {
		fprintf (stderr, "Read failed\n");
		return -1;
	}	
	len = ((int) je16_to_cpu (rec->hdr.len)) - sizeof (struct jvisu_cmdhdr);
	timeo.tv_sec = 0;
	timeo.tv_usec = 500000;
	if (select(sock + 1, &rfds, NULL, NULL, &timeo) <= 0) {
		fprintf (stderr, "Select failed\n");
		return -1;
	}
	if (read (sock, &buf[sizeof (struct jvisu_cmdhdr)], len) != len) {
		fprintf (stderr, "Read failed\n");
		return -1;
	}
		
	switch (rec->hdr.type) {
	case VAR_TYPE_DOUBLE: {
		uint64_t tmp = je64_to_cpu (rec->data.d[0]);
		sprintf (value, "%f", *((double *) &tmp));
		break;
	}	
	case VAR_TYPE_FLOAT: {
		uint32_t tmp = je32_to_cpu (rec->data.i[0]);
		sprintf (value, "%f", *((float *) &tmp));
		break;
	}	
	case VAR_TYPE_INT:
		sprintf (value, "%d", (int) je32_to_cpu (rec->data.i[0]));
		break;
	case VAR_TYPE_SHORT:
		sprintf (value, "%d", (short) je16_to_cpu (rec->data.s[0]));
		break;
	case VAR_TYPE_BOOL:
	case VAR_TYPE_BYTE:
		sprintf (value, "%d", rec->data.b[0]);
		break;
	case VAR_TYPE_STRING:
		strcpy (value, rec->data.b); 
		break;
	default:
		return -1;
	}
	return 0;
}

/*
*	Try to do this as simple as possible
*/
int main (int argc, char *argv[])
{
	char			buf[256];
	struct jvisu_regvar	*var = (struct jvisu_regvar *) buf;
	struct jvisu_sndvar	*snd = (struct jvisu_sndvar *) buf;
	struct sockaddr_in 	saddr;
	struct hostent 		*hostinfo;
	int			sock = -1, len, res = 0;

	process_options (argc, argv);
	
	// try to connect to JVisu server
	hostinfo = gethostbyname ("localhost");
	if (hostinfo == NULL) {
		fprintf (stderr, "Gethostby name failed\n");
		goto err;
	}	

	saddr.sin_addr = *(struct in_addr *)hostinfo->h_addr;
	saddr.sin_family = AF_INET;
	saddr.sin_port = htons (port);

	// create socket
	sock = socket (AF_INET, SOCK_STREAM, 0);
	if (sock < 0) {
		fprintf (stderr, "Create socket failed\n");
		goto err;
	}		
	// connect
	if (connect (sock, (struct sockaddr *) &saddr, sizeof (saddr)) < 0) {
		fprintf (stderr, "Connect failed\n");
		goto err;
	}

	// Register variable 	
	strcpy (var->symbol, symbol);
	var->idx = cpu_to_je32 (0);
	var->hdr.type = (u_char) vartype;
	var->rate = cpu_to_je32 (100000);
	var->hdr.cmd = CMD_REG_VAR;

	len = sizeof (struct jvisu_regvar) + strlen (var->symbol);
	var->hdr.len = cpu_to_je16 ((ushort) len);
	if (write (sock, buf, len) != len) {
		fprintf (stderr, "Write to socket failed\n");
		goto err;
	}
	if (read_result (sock))
		goto err;
		
	if (!writevar)
		goto out;
	
	/* Send variable value */	
	snd->idx = cpu_to_je32 (0);
	snd->hdr.cmd = CMD_SEND_VAR;
	snd->hdr.type = (char) vartype;
	switch (vartype) {
	case VAR_TYPE_DOUBLE: {
		double   d;
		uint64_t *p = (uint64_t *) &d;
		d = strtod (writevalue, NULL);
		snd->data.d[0] = cpu_to_je64 (*p);
		len = sizeof (uint64_t);
		break;
	}	
	case VAR_TYPE_FLOAT: {
		float   d;
		uint32_t *p = (uint32_t *) &d;
		d = (float) strtod (writevalue, NULL);
		snd->data.i[0] = cpu_to_je32 (*p);
		len = sizeof (uint32_t);
		break;
	}	
	case VAR_TYPE_INT: {
		int   d;
		uint32_t *p = (uint32_t *) &d;
		d = atoi (writevalue);
		snd->data.i[0] = cpu_to_je32 (*p);
		len = sizeof (uint32_t);
		break;
	}
	case VAR_TYPE_SHORT: {
		short d;
		uint16_t *p = (uint16_t *) &d;
		d = (short) atoi (writevalue);
		snd->data.s[0] = cpu_to_je16 (*p);
		len = sizeof (uint16_t);
		break;
	}	
	case VAR_TYPE_BOOL:
	case VAR_TYPE_BYTE:
		snd->data.b[0] = (char) atoi (writevalue);
		len = sizeof (char);
		break;
	case VAR_TYPE_STRING:
		strcpy (snd->data.b, writevalue);
		len = strlen (writevalue);
		break;
	default:
		goto err;
	}
	
	len += sizeof (struct jvisu_sndvar);
	snd->hdr.len = cpu_to_je16 ((short) len);
	if (write (sock, buf, len) != len) {
		fprintf (stderr, "Write to socket failed\n");
		goto err;
	}
	if (read_result (sock))
		goto err;
			
err:
	res = -1;
out:
	if (sock >= 0)
		close (sock);
	fprintf (stdout, "%s\n", value);
	exit (res);	
}
