package org.jconfig.items;

import java.lang.*;
import java.util.*;

/**
 *  <p>package:    org.jconfig.items</p>
 *  <p>file:       TreeItem.java</p>
 * $Id: TreeItem.java,v 1.5 2004/04/19 08:12:45 tgl Exp $
 *
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 * <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.</p>
 * </p>
 *
 * Change Log:
 * </p>
 * <ul>
 * <li> 1.0 First implementation
 * </ul>
 * <p>
 *	TreeItem is a storage class for a XML based configuration tree
 *	which is shown in a JTree 
 * </p>
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.0
 */
public class TreeItem extends java.lang.Object {
    
    private TreeItem    parent, config;
    private String      name;
    private String      content="";
    public  ArrayList   items, attr, text;
    public  int         created = 0;
    
    public static final int TREEITEM_ITEM = 1;
    public static final int TREEITEM_ATTR = 2;
    public static final int TREEITEM_TEXT = 3;
    
    /** Creates a new instance of TreeItem 
     */
    public TreeItem() {
        initItem ("", null, null);
    }

    /** Creates a new instance of TreeItem 
     *  @param nam name of the item
     */
    public TreeItem(String nam) {
        initItem (nam, null, null);
    }

    /** Creates a new instance of TreeItem 
     *  @param nam name of the item
     *  @param par parent item
     */
    public TreeItem(String nam, TreeItem par) {
        initItem (nam, par, null);
    }

    /** Creates a new instance of TreeItem 
     *  @param nam name of the item
     *  @param par parent item
     *  @param conf configuration item
     */
    public TreeItem(String nam, TreeItem par, TreeItem conf) {
        initItem (nam, par, conf);
    }

    /* Create an array item 
     *  @param it item to clone
     *  @param index. If index < 0 no attributes and text is modified
     */
    public TreeItem(TreeItem it, int index) {

        initItem (it.GetName(), it.GetParent(), it.GetConfig());
        /* Clone attributes */
        for (int i = 0; i < it.attr.size(); i++) {
            ItemAttr att = (ItemAttr) it.attr.get(i);
            AddAttr (new ItemAttr (att.getName(), att.getValue()));
        }    
        /* Clone subitems */
        for (int i = 0; i < it.items.size(); i++)
            AddItem (new TreeItem ((TreeItem) it.items.get(i), -1));
        
        /* Clone content */
        SetContent (it.GetContent());
        
        /* Check, if we have to modify attributes and text */
        if (index < 0)
            return;
        
        /* Get autoincr string */
        String txt = GetAttrValue ("autoincr");
        if (txt != null) {
            String[] tag = txt.split(",");
            for (int i = 0; i < tag.length; i++) {
                txt = GetAttrValue (tag[i]);
                if (txt != null)
                    ModifyAttr(tag[i], txt + index);
            }    
        }
        
        for (int i = 0; i < text.size(); i++) {
            TreeItem tit = (TreeItem) text.get(i);
            tit.SetContent(tit.GetContent() + " " + index);
        }
    }

    /** Initialize item 
     *  @param nam name of the item
     *  @param par parent item
     *  @param conf configuration item
     */
    private void initItem (String nam, TreeItem par, TreeItem conf) {
        name = new String(nam);
        parent = par;
        config = conf;
        items = new ArrayList();
        text = new ArrayList();
        attr = new ArrayList();
    }
    
    /** Get the content 
     *  @return content
     */
    public String GetContent () {
        return new String(content);
    }

    /** Set the content 
     *  @param c new content 
     */
    public void SetContent (String c) {
        // Remove whitespace
        content = new String(c.trim());
    }

    /** Get the configuration item
     * @return config item
     */
    public TreeItem GetConfig () {
        return config;
    }

    /** Set the configuration item
     *  @param newconfig the new configuration item
     *  @return previous configuration item
     */
    public TreeItem SetConfig (TreeItem newconfig) {
        config = newconfig;
        return config;
    }

    /* Get the name of the item
     * @return name
     */
    public String GetName () {
        return new String(name);
    }

    /* Compare the name with a given String
     * @param n string to compare 
     * @return true, if Strings are equal
     */
    public boolean HasName (String n) {
        if (name == null || n == null)
            return false;
        return (name.compareTo(n) == 0) ? true : false;
    }
    
    /** Item handling */
    /** Add a new Item. Set Parent
     *  @param item item to add
     *  @return added item
     */
    public TreeItem AddItem (TreeItem item) {
        items.add(item);
        // Special handling for textitems
        if (item.GetName().compareTo("textitem") == 0)
            text.add (item);
        item.SetParent(this);
        return item;
    }
    
    /** Create and insert item
     *  @param item item to insert
     *  @return item or null, if insertion not possible
     */
    public TreeItem InsertItem (TreeItem item) {
        return InsertItem (item, true);
    }
    
    /** Insert item
     *  @param item item to insert
     *  @param create if ture create a new item
     *  @return item or null, if insertion not possible
     */
    public TreeItem InsertItem (TreeItem item, boolean create) {
        // If no configuration given, return
        if (config == null)
            return null;
        // Check, if this item is configured as unique
        String cmp = item.GetAttrValue("unique");
        if (cmp != null && cmp.compareTo("true") == 0) {
            if (GetItemFollow (item.name) != null)
                return null;
        }
        // Can you insert optional items ?
        cmp = config.GetAttrValue ("optitem");
        if (cmp == null)
            return null;
        String[] valid = cmp.split(",");
        for (int i = 0; i < valid.length; i++) {
            // Is the new item allowed to insert ?
            if (valid[i].compareTo(item.name) == 0) {
                // Whee it worked
                TreeItem newit;
                if (create) {
                    newit = new TreeItem (item.name, this, item);
                    if (newit == null)
                        return null;
                    newit.AddAttr(new ItemAttr("name", item.name + item.created));
                    item.created++;
                    for (int j = 0; j < item.items.size(); j++) {
                        TreeItem subitem  = (TreeItem) item.items.get(j);
                        String def = subitem.GetAttrValue("default");
                        if (def != null)
                            newit.ModifyAttr(subitem.GetAttrValue("name"), def);
                    }
                } else 
                    newit = item;
                
                AddItem(newit);
                return newit;
            } 
        }    
        return null;
    }
    
    /** Return a item in the item array
     *  @param i index in array
     *  @return item or null, if no item available
     */
    public TreeItem GetItem (int i) {
        if (i < items.size()) 
            return (TreeItem) items.get(i);
        else
            return null;
    }
    
    /** Return a item with a given name. Search recursive
     *  @param n name to compare
     *  @return item or null, if no item available
     */
    public TreeItem GetItemFollow (String n) {
        if (name.compareTo(n) == 0)
            return this;
        
        for (int i = 0; i < items.size(); i++) {
            TreeItem it = ((TreeItem)items.get(i)).GetItemFollow(n);
            if (it != null)
                return it;
        }
        return null;
    }

    /** Search a TreeItem with the given name and attributes
     *  @param n name to compare
     *  @param attr array of attributes ot compare
     *  @return item or null, if no item available
     */ 
    public TreeItem GetItemFollow (String n, ItemAttr attr[]) {
        return GetItemFollow(n, attr, null);
    }

    /** Search a TreeItem with the given name and attributes and
     *  matching content
     *  @param n name to compare
     *  @param attr array of attributes ot compare
     *  @param cont content to compare. if cont is null no comparison happens
     *  @return item or null, if no item available
     */ 
    public TreeItem GetItemFollow (String n, ItemAttr attr[], String cont) {
        TreeItem it = null;
        
        if (name.compareTo(n) == 0) {
            it = this;
            for (int i = 0; i < attr.length; i++) {
                if (attr[i] != null) {
                    if (CheckAttrValue(attr[i]) == false)
                        it = null;
                }        
            }
        }
        
        if (it != null) {
            if (cont == null)
                return it;
            if (cont.compareTo(it.GetContent()) == 0)
                return it;
        }
        
        for (int i = 0; i < items.size(); i++) {
            TreeItem subit = ((TreeItem)items.get(i)).GetItemFollow(n, attr, cont);
            if (subit != null)
                return subit;
        }
        return null;
    }

    /** Return the items array 
     */
    public Object[] GetItemsArray () {
        return items.toArray();
    }
    
    /** Remove the item from the array
     */
    public void RemoveItem (TreeItem item) {
        items.remove (item);
    }
    
    /** Remove the item from the parent array
     */
    public TreeItem RemoveFromParent () {
        if (parent == null)
            return null;
        parent.RemoveItem(this);
        return parent;
    }
    
    /** Remove all items recursive
     */
    public void RemoveAll() {
        while(attr.size() > 0)
            attr.remove(attr.size()-1);
        
        while(text.size() > 0)
            text.remove(text.size()-1);
        
        while (true) {
            if (items.size() == 0)
                break;
            ((TreeItem)items.get(items.size()-1)).RemoveAll();
        }
        RemoveFromParent ();
    }

    /** Return parent item
     */
    public TreeItem GetParent () {
        return parent;
    }

    /** Set parent item
     */
    public TreeItem SetParent (TreeItem newparent) {
        parent = newparent;
        return parent;
    }

    /** Add Attributes 
     */
    public void AddAttr (ItemAttr attrib) {
        attr.add(attrib);
    }
    
    /** Modify an attribute value. If the attribute does not
     * exists, create a new attribute and set the value
     * @param attrname Name of the attribute
     * @param value new value of the attribute
     */
    public void ModifyAttr (String attrname, String value) {
        ItemAttr att = GetAttr (attrname);
        if (att != null)
            att.setValue(value);
        else {
            att = new ItemAttr (attrname, value);
            attr.add (att);
        }
    }

    /** Return item attribute
     */
    public ItemAttr GetAttr (int index) {
        if (index < attr.size())
            return (ItemAttr)attr.get(index);
        return null;
    }
    
    /** Return item attribute with given name
     */
    public ItemAttr GetAttr (String attrname) {
        for (int i = 0; i < attr.size(); i++) {
            ItemAttr att = (ItemAttr)attr.get(i);
            if (att.getName().compareTo(attrname) == 0)
                return att;
        }
        return null;
    }

    /** Return attribute array
     */
    public Object[] GetAttrArray () {
        return attr.toArray();
    }

    /** Return attribute value for attribute with given name
     */
    public String GetAttrValue (String attrname) {
        ItemAttr att = GetAttr (attrname);
        if (att != null)
            return att.getValue();
        return null;
    }

    /** Return attribute value for attribute at index 
     */
    public String GetAttrValue (int index) {
        if (index < attr.size()) {
            ItemAttr att = (ItemAttr)attr.get(index);
            return att.getValue();
        }    
        return null;
    }

    /** Check, if the attribute value is identical
     */
    public boolean CheckAttrValue (ItemAttr attr) {
        return CheckAttrValue (attr.getName(), attr.getValue());
    }

    /** Check, if the attribute value is identical
     */
    public boolean CheckAttrValue (String attrname, String attrvalue) {
        ItemAttr att = GetAttr (attrname);
        if (att == null || att.getValue() == null || att.getValue().compareTo(attrvalue) != 0)
            return false;
        return true;
    }

    /** get attribute value for a given name
     *  if not found create an empty one
     */
    public String GetOrBuildAttrValue (String attrname) {
        ItemAttr att = GetAttr (attrname);
        if (att != null)
            return att.getValue();
        attr.add (new ItemAttr (attrname, ""));
        return ("");
    }

    /** get attribute value for a given name
     *  if not found create one with the given default value
     */
    public String GetOrBuildAttrValue (String attrname, String defvalue) {
        ItemAttr att = GetAttr (attrname);
        if (att != null)
            return att.getValue();
        attr.add (new ItemAttr (attrname, defvalue));
        return (defvalue);
    }

    public ItemToXML printFollow (ItemToXML encoder) {
        return printFollow(encoder, true, true);
    }

    public ItemToXML printFollow (ItemToXML encoder, boolean hasconfig) {
        return printFollow (encoder, hasconfig, false);
    }

    public ItemToXML printFollow (ItemToXML encoder, boolean hasconfig, boolean savearray) {
        if (config == null && hasconfig)
            return encoder;
        
        if (encoder == null)
            encoder = new ItemToXML();

        encoder.StartTag(this, savearray);
        for (int i = 0; i < items.size(); i++)
            ((TreeItem)items.get(i)).printFollow(encoder, hasconfig, savearray);
        encoder.EndTag(this, items.size() > 0 ? true : false);
        return encoder;
    }
    
    public String toString () {
        if (text.size() == 0) {
            String txt = GetAttrValue("object");
            if (txt != null)
                return name + ": " + txt;
            else
                return name;
        }
        
        int i = ItemLanguage.language;
        // if this language is not available use default language
        if (i >= text.size())
            i = 0;
        return ((TreeItem) text.get(i)).GetContent();
    }
}
