package org.jvisu.panel.configurator;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.*;
import javax.swing.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;
import org.jvisu.panel.*;
import org.jvisu.util.*;
import org.jvisu.util.param.*;
/**
 *  <p>package:    org.jvisu.panel.configurator</p>
 *  <p>file:       JVPalettePanel.java</p>
 * $Id: JVPalettePanel.java,v 1.7 2004/04/19 08:20:55 tgl Exp $
 *
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 * <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.</p>
 * </p>
 *
 * Change Log:
 * </p>
 * <ul>
 * <li> 1.0 First implementation
 * </ul>
 * <p>
 *    Panel to hold the widget palette. Read in from XML file
 * </p>
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.3
 */
public class JVPalettePanel extends javax.swing.JTabbedPane implements ActionListener {
    
    /** Button group to have only one selected at a time*/
    private ButtonGroup     buttons;
    /** Actionmap */
    private ActionMap       actions;
    /** The selecor button */
    private JToggleButton   selectButton;
    /** The seleced button */
    private JToggleButton   selected;
    /** Standard panel font */
    private Font            font;
    
    /** Creates a new instance of JVPalettePanel */
    public JVPalettePanel() {
        buttons = new ButtonGroup();
        actions = new ActionMap();
        selectButton = new JToggleButton();
        selectButton.addActionListener(this);
        buttons.add(selectButton);
        font = new Font("Dialog", Font.PLAIN, 10);
        setFont(new Font("Dialog", Font.PLAIN, 12));
    }
    
    /** Somebody pressed a button
     */
    public void actionPerformed(java.awt.event.ActionEvent e) {
        try {
            JToggleButton old = selected;
            selected = (JToggleButton) e.getSource();
            if (selected != old)
                this.firePropertyChange("selectedElement", old.getAction(), selected.getAction());
            
        } catch (Exception ex) { }
    }
    
    /** Get actionmap
     *  @return action map
     */
    public ActionMap getActions() {
        return actions;
    }
    
    /** Add a component to a certain category.
     *  if the category does not exist create a new one
     *  @param action   action, which should be added
     *  @param tocat    category, to which the component should be added
     */
    public void addCompToCategory(JVPaletteAction action, String tocat) {
        JPanel panel = null;
        for (int i = 0; i < this.getTabCount(); i++) {
            if (this.getTitleAt(i).compareTo(tocat) == 0) {
                JScrollPane pane = (JScrollPane)this.getComponentAt(i);
                try {
                    panel = (JPanel)pane.getViewport().getComponent(0);
                } catch (ClassCastException e) { }
                break;
            }
        }
        if (panel == null) {
            panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
            JScrollPane pane = new JScrollPane();
            pane.setMinimumSize(new Dimension(300, 50));
            pane.setViewportView(panel);
            this.addTab(tocat, pane);
        }
        
        actions.put(action.getDesignModel(), action);
        if (!action.isHidden()) {
            JToggleButton but = new JToggleButton(action);
            but.setFont(font);
            but.addActionListener(this);
            buttons.add(but);
            panel.add(but);
        }
    }
    
    /** Remove the selection
     */
    public void clearSelection() {
        selectButton.doClick();
    }
    
    /** Return the action, which is associated with the selected item
     */
    public JVPaletteAction getSelection() {
        return selectButton.isSelected() ? null : (JVPaletteAction) selected.getAction();
    }
    
    /** Return the action, which is associated with the key
     *  @param model he designmodel for this component
     *  @return the action associated with this component
     */
    public JVPaletteAction getActionForDesignModel(String model) {
        return (JVPaletteAction) actions.get(model);
    }
    
    /** Read a palette configuration file
     *  @param file the file, which msut be parsed
     */
    public void parsePaletteFile(URL file) throws Exception {
        
        JVPalettePanel.PaletteItemHandler handler = new JVPalettePanel.PaletteItemHandler();
        handler.setPanel(this);
        URLConnection ctrlConnection = file.openConnection();
        InputStream in = ctrlConnection.getInputStream();
        SAXParser p = (SAXParserFactory.newInstance()).newSAXParser();
        p.parse(in, handler);
        in.close();
        clearSelection();
    }
    
    /** The class for palette actions
     *  This class is used to implement the palette buttons and
     *  to provide information about the component
     *
     */
    public class JVPaletteAction extends AbstractAction {
        
        /** The category name */
        private String          category;
        /** The components class name */
        private String          designmodel;
        /** The components xml representaion */
        private String          xmlname;
        /** The action, which provides the basic information for this action*/
        private JVPaletteAction superaction;
        /** The reference component for this action */
        private JComponent      reference;
        /** The properties, which are given for this component */
        private ArrayList       properties;
        /** Hidden action, which is used to store the Panel information */
        private boolean         hidden;
        
        /** create a new instance
         *  @param cat the category of this action
         */
        public JVPaletteAction(String cat) {
            properties = new ArrayList();
        }
        
        /** Somebody pressed the button
         */
        public void actionPerformed(java.awt.event.ActionEvent e) {
        }
        
        /** return the setting of hidden
         *  @return true, if this action is hidden
         */
        public boolean isHidden() {
            return hidden;
        }
        
        /** Set the hidden value
         *  @param ishidden new value of hidden
         */
        public void setHidden(boolean ishidden) {
            hidden = ishidden;
        }
        
        /** Get the action, which provides te basic information for this action
         *  @return the action, which has the basic information for this component
         */
        public JVPaletteAction getSuperAction() {
            return superaction;
        }
        
        /** Set the action, which provides te basic information for this action
         *  @param action the action, which has the basic information for this component
         */
        public void setSuperAction(JVPaletteAction action) {
            superaction = action;
        }
        
        /** Get the components xml tag name
         *  @return the xml tagname of this component
         */
        public String getXMLTagName() {
            if (superaction == null)
                return xmlname;
            return superaction.getXMLTagName();
        }
        
        /** Get the components xml tag name
         *  @param name the xml tagname of this component
         */
        public void setXMLTagName(String name) {
            xmlname = name;
        }
        
        /** Get the components designmodel
         *  @return the designmodel of thios component
         */
        public String getDesignModel() {
            return designmodel;
        }
        
        /** Get the components base designmodel
         *  This is the components designmodel, if this component has no
         *  superaction. If the component has a superaction, then the
         *  superclass designmodel is returned
         *  @return the base designmodel of thios component
         */
        public String getBaseDesignModel() {
            if (superaction == null)
                return designmodel;
            return superaction.getBaseDesignModel();
        }
        
        /** Set the components designmodel
         *  @param model the designmodel of this component
         */
        public void setDesignModel(String model) {
            designmodel = model;
            reference = JVBasePanel.createComponent(model);
        }
        
        /** Set the reference component
         *  @param comp either an instance of the reference component or null
         */
        public void setRefComponent(JComponent comp) {
            reference = comp;
        }
        
        /** Get the reference component
         *  @return the reference component from this action or from the
         *          given super action, or null if none exists
         */
        public JComponent getRefComponent() {
            if (superaction != null)
                return superaction.getRefComponent();
            return reference;
        }
        
        /** Add a property object to the arrylist
         *  @param prop propertyobject, which has to be added
         */
        public void addProperty(Object prop) {
            properties.add(prop);
        }
        
        /** Get the properties array
         *  If this component has a superaction, then return the
         *  properties of the superaction at the beginning of the array
         *  @return array of the properties
         */
        public ArrayList getProperties() {
            ArrayList props = new ArrayList();
            if (superaction != null) {
                ArrayList sap = superaction.getProperties();
                for (int i = 0; i < sap.size(); i++)
                    props.add(sap.get(i));
            }
            for (int i = 0; i < properties.size(); i++)
                props.add(properties.get(i));
            return props;
        }
    }
    
    /** A class to hold item properties */
    public class PaletteItemProperty extends Object {
        
        /** set true, if this property is fixed and cannot be changed */
        private boolean     fixed;
        /** set true, if this property is expert level */
        private boolean     expert;
        /** property name */
        private String      name;
        /** property value */
        private String      value;
        /** property category */
        private String      category;
        /** property editor */
        private String      editor;
        /** Storage for text definitions */
        private ArrayList   text;
        
        /** Create a new instance of properties */
        public PaletteItemProperty() {
            text = new ArrayList();
        }
        
        /** return the setting of fixed
         *  @return true, if this property is fixed
         */
        public boolean isFixed() {
            return fixed;
        }
        
        /** Set the fixed value
         *  @param isfixed new value of fixed
         */
        public void setFixed(boolean isfixed) {
            fixed = isfixed;
        }
        
        /** return the setting of expert
         *  @return true, if this property is expert level
         */
        public boolean isExpert() {
            return expert;
        }
        
        /** Set the expert value
         *  @param isexpert new value of expert
         */
        public void setExpert(boolean isexpert) {
            expert = isexpert;
        }
        
        /** Get the property name
         *  @return property name
         */
        public String getName() {
            return name;
        }
        
        /** Set the property name
         *  @param n name of the property
         */
        public void setName(String n) {
            name = n;
        }
        
        /** Get the property value
         *  @return property vlaue
         */
        public String getValue() {
            return value;
        }
        
        /** Set the property value
         *  @param v value of the property
         */
        public void setValue(String v) {
            value = v;
        }
        
        /** Get the property category
         *  @return property category
         */
        public String getCategory() {
            return category;
        }
        
        /** Set the property category
         *  @param c category of the property
         */
        public void setCategory(String c) {
            category = c;
        }
        
        /** Get the property editor
         *  @return property editor
         */
        public String getEditor() {
            return editor;
        }
        
        /** Set the property editor
         *  @param c editor of the property
         */
        public void setEditor(String c) {
            editor = c;
        }
        
        /** Set the text for a specific language
         *  Copy the default entry, if there is a gap
         *  @param t    text
         *  @param idx language index
         */
        public void setTextForLanguage(String t, int idx) {
            /* Replace an existing entry ? */
            if (idx < text.size()) {
                text.set(idx, t);
            } else {
                for (int i = text.size(); i < idx; i++)
                    text.add(text.get(0));
                text.add(t);
            }
        }
        
        /** Return the String representation of this
         *  property. If no texts are available return the
         *  name of the property. If no text is available
         *  for the property, then return the default text
         *
         *  @return String representation of the property
         */
        public String toString() {
            if (text.size() == 0)
                return name;
            try {
                return (String) text.get(JVLanguageSelector.getLanguageSelector());
            } catch (IndexOutOfBoundsException e) {
                return (String) text.get(0);
            }
        }
    }
    
    /** The xml handler for palette configuration
     */
    private class PaletteItemHandler extends org.xml.sax.helpers.DefaultHandler {
        
        /** The current indentation level */
        private int                 level = 0;
        /** The palette */
        private JVPalettePanel      panel;
        /** The category */
        private String              category;
        /** JVPaletteAction */
        private JVPaletteAction     action;
        /** current qName */
        private String              qname;
        /** property    */
        private PaletteItemProperty property;
        /** language of text */
        private int                 language = 0;
        /** text content */
        private String              content;
        
        /** The palette panel, which must be configured */
        
        /** Creates a new instance of ItemHandler */
        public PaletteItemHandler() {
        }
        
        /** Set the palette panel
         *  @param panel    panel to add items
         */
        public void setPanel(JVPalettePanel pan) {
            panel = pan;
        }
        
        /** Decode the content */
        private String Decode(String content) {
            try {
                return java.net.URLDecoder.decode(content, "UTF-8");
            } catch (Exception e) {
                System.out.println(e);
                System.out.println(content);
            }
            return content;
        }
        
        public void startElement(String uri, String localName, String qName, Attributes attributes) {
            qname = qName;
            level++;
            if (qName.equals("category")) {
                int idx = attributes.getIndex("name");
                if (idx != -1)
                    category = Decode(attributes.getValue(idx));
            }
            
            if (qName.equals("component")) {
                SplashScreen.advanceSplashProgress();
                action = new JVPalettePanel.JVPaletteAction(category);
                for (int i = 0; i < attributes.getLength(); i++) {
                    String name = Decode(attributes.getQName(i));
                    String value = Decode(attributes.getValue(i));
                    if (name.equals("object")) {
                        action.setDesignModel(value);
                        continue;
                    }
                    if (name.equals("xmltagname")) {
                        action.setXMLTagName(value);
                        continue;
                    }
                    if (name.equals("hidden")) {
                        action.setHidden(value.equalsIgnoreCase("true"));
                        continue;
                    }
                    if (name.equals("superclass")) {
                        if (value.length() == 0)
                            continue;
                        action.setSuperAction((JVPaletteAction) panel.getActions().get(value));
                        continue;
                    }
                    if (name.equals("name")) {
                        action.putValue(AbstractAction.NAME, value);
                        continue;
                    }
                    if (name.equals("description")) {
                        action.putValue(AbstractAction.SHORT_DESCRIPTION, value);
                        continue;
                    }
                }
            }
            
            if (qname.equals("property")) {
                property = new JVPalettePanel.PaletteItemProperty();
                for (int i = 0; i < attributes.getLength(); i++) {
                    String name = Decode(attributes.getQName(i));
                    String value = Decode(attributes.getValue(i));
                    if (name.equals("name")) {
                        property.setName(value);
                        continue;
                    }
                    if (name.equals("value")) {
                        if (value.length() > 0)
                            property.setValue(value);
                        continue;
                    }
                    if (name.equals("fixed")) {
                        property.setFixed(value.equalsIgnoreCase("true"));
                        continue;
                    }
                    if (name.equals("expert")) {
                        property.setExpert(value.equalsIgnoreCase("true"));
                        continue;
                    }
                    if (name.equals("category")) {
                        property.setCategory(value);
                        continue;
                    }
                    if (name.equals("editor")) {
                        property.setEditor(value);
                        continue;
                    }
                }
            }
            
            if (qname.equals("text")) {
                int idx = attributes.getIndex("lan");
                if (idx != -1)
                    language = JVParams.StringToInt(
                    Decode(attributes.getValue(idx)), -1);
            }
        }
        
        public void endElement(String uri, String localName, String qName) {
            if (qName.equals("component"))
                panel.addCompToCategory(action, category);
            if (qName.equals("property")) {
                if (property != null)
                    action.addProperty(property);
            }
            if (qName.equals("text")) {
                //System.out.println ("Lan: " + language + ": " + content);
                property.setTextForLanguage(content, language);
            }
            if (qName.equals("image")) {
                try {
                    URL url = new URL(content);
                    action.putValue(AbstractAction.SMALL_ICON, new ImageIcon(url));
                } catch (Exception e) {
                    try {
                        action.putValue(AbstractAction.SMALL_ICON,
                        new ImageIcon(getClass().getResource(content)));
                    } catch (Exception e1) {}
                }
            }
        }
        
        public void processingInstruction(String target, String data) {
            System.out.println("processing instruction: " + target);
        }
        
        public void characters(char[] ch, int start, int length) {
            
            String c = Decode(new String(ch, start, length));
            content = c;
        }
        
        public void warning(SAXParseException e) {
            System.out.println("Warning:" + (e.getMessage() != null
            ? e.getMessage()
            : e.getClass().getName()));
        }
        
        public void error(SAXParseException e) {
            System.out.println("Error:" + (e.getMessage() != null
            ? e.getMessage()
            : e.getClass().getName()));
        }
    }
}
